@@ -4,18 +4,21 @@
 import os
 import glob
 
+from ..base import *
 from ..utility import *
 
 
 class PageIterator:
 
-    def __init__(self, xml_directory):
-        self.path_list = sorted(glob.glob(os.path.join(xml_directory, "*.xml")))
+    def __init__(self, name = None):
+        self.config = Config(name)
+        self.xml_directory = self.config.get_wikipedia_xml_directory(must = True)
+
+    def __iter__(self):
+        self.path_list = sorted(glob.glob(os.path.join(self.xml_directory, "*.xml")))
         self.n_path = len(self.path_list)
         self.i_path = 0
         self.page_list = []
-
-    def __iter__(self):
         return self
 
     def __next__(self):
