@@ -12,27 +12,14 @@ class Constant:
     root_config = os.path.join(root_directory, "config.yml")
     root_status = os.path.join(root_directory, "status.yml")
 
-    task_directory = os.path.join(root_directory, "task")
     wikipedia_directory = os.path.join(root_directory, "wikipedia")
 
+    name_format = r"^[a-zA-Z0-9_.\-]+$"
+
     available_languages = [
         "japanese",
         "english"
     ]
 
-    available_algorithms = [
-        "word2vec",
-        "doc2vec",
-        "word_frequency"
-    ]
-
-    available_tokenizers = [
-        "mecab",
-        "janome",
-        "nltk"
-    ]
-
     min_page_chunk = 100
     max_page_chunk = 10 ** 7
-    min_worker = 1
-    max_worker = 64
